import { __awaiter, __decorate } from "tslib";
import { FetchMachine } from '@genesislcap/foundation-state-machine';
import { AbstractStore, registerStore } from '@genesislcap/foundation-store';
import { observable, volatile } from '@microsoft/fast-element';
import { AuthConfig } from '../config/config';
import { CredentialManager, credentialSeparator } from '../credential';
import { fetchIDPs, logger } from '../utils';
/**
 * Default implementation of the store.
 * @public
 */
export class DefaultAuthFormEntryStore extends AbstractStore {
    get orgUsername() {
        return this.config.fields.organisation && this.organisation.trim() !== ''
            ? `${this.username.trim()}:${this.organisation.trim()}`
            : this.username.trim();
    }
    constructor() {
        var _a, _b;
        super();
        this.host = '';
        this.organisation = '';
        this.username = '';
        this.email = '';
        this.password = '';
        this.password2 = ''; // < New, confirm etc.
        this.showPasswords = false;
        this.hasPasswordExpired = false;
        this.passwordResetToken = '';
        this.message = '';
        this.hasErrors = false;
        this.mfaCode = '';
        this.idps = [];
        this.selectedIDP = '';
        this.ssoToggled = false;
        this.termsAgreed = false;
        this.autofilling = false;
        this.onAutofillForm = this.createListener('auth-autofill-form', () => this.autofill());
        this.onOrganisationChanged = this.createListener('auth-organisation-changed', (detail) => (this.commit.organisation = detail));
        this.onUsernameChanged = this.createListener('auth-username-changed', (detail) => (this.commit.username = detail));
        this.onEmailChanged = this.createListener('auth-email-changed', (detail) => (this.commit.email = detail));
        this.onPasswordChanged = this.createListener('auth-password-changed', (detail) => (this.commit.password = detail));
        this.onPassword2Changed = this.createListener('auth-password2-changed', (detail) => (this.commit.password2 = detail));
        this.onShowPasswordsChanged = this.createListener('auth-showPasswords-changed', (detail) => (this.commit.showPasswords = detail));
        this.onSelectedIDPChanged = this.createListener('auth-selectedIDP-changed', (detail) => (this.commit.selectedIDP = detail));
        this.onSSOToggleChanged = this.createListener('auth-ssoToggled-changed', (detail) => (this.commit.ssoToggled = detail));
        this.ssoToggled = (_b = (_a = this.config.sso) === null || _a === void 0 ? void 0 : _a.toggled) !== null && _b !== void 0 ? _b : false;
        /**
         * Listeners not on the public interface can be created anonymously
         */
        this.createListener('auth-fetch-idps-success', (idps) => {
            this.commit.idps = idps;
            this.commit.selectedIDP = idps && idps.length ? idps[0].id : '';
        });
        this.createErrorListener('auth-fetch-idps-error', () => this.resetIDPState());
    }
    resetIDPState() {
        this.commit.idps = [];
        this.commit.selectedIDP = '';
    }
    autofill() {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            let credentialData;
            try {
                /**
                 * TODO: Revert back to `credentialManager.getCredentials()` after Neptune deployment.
                 * Neptune was using cookies directly. To provide them with a seamless upgrade, we should load from cookies if the
                 * container is empty. These stored cookies become irrelevant after their first login in the new system.
                 */
                this.autofilling = true;
                const containerData = yield this.credentialManager.getCredentialsFromContainer();
                const cookieData = yield this.credentialManager.getCredentialsFromCookie();
                const [normalisedData, canRemoveCookie] = this.credentialManager.normaliseLegacyCredentialData(containerData, cookieData);
                if (cookieData && canRemoveCookie) {
                    /**
                     * Upgrade path: They have chosen a profile from the container, yet also have a cookie... it's safe to remove the cookie.
                     */
                    this.credentialManager.removeCookie();
                }
                credentialData = normalisedData;
                if (credentialData) {
                    const passwordCredentials = credentialData;
                    this.username = (_a = passwordCredentials.name) !== null && _a !== void 0 ? _a : '';
                    this.password = (_b = passwordCredentials.password) !== null && _b !== void 0 ? _b : '';
                    if (this.config.fields.organisation) {
                        const combinedCompIdUsername = (_c = passwordCredentials.id) !== null && _c !== void 0 ? _c : '';
                        const valueParts = combinedCompIdUsername.split(credentialSeparator);
                        const orgIdx = valueParts.length === 2 ? 1 : 0;
                        const proposedOrganisation = valueParts[orgIdx];
                        if (proposedOrganisation !== this.username) {
                            this.organisation = proposedOrganisation;
                        }
                    }
                }
            }
            catch (err) {
                logger.error('An error occurred attempting to get and autofill from stored credentials.', err);
            }
            this.autofilling = false;
            return Promise.resolve(!!credentialData);
        });
    }
    ssoToggledChanged(prev, next) {
        var _a, _b;
        if (!next) {
            this.resetIDPState();
            return;
        }
        const path = `${(_b = (_a = this.config.sso) === null || _a === void 0 ? void 0 : _a.identityProvidersPath) !== null && _b !== void 0 ? _b : 'sso/list'}`;
        this.errors.delete('auth-fetch-idps-error');
        const emitError = () => this.emit('auth-fetch-idps-error', Error('Failed to fetch identity providers.'));
        /**
         * TODO: Port to FetchMachine, need to sort out all the utils/sso stuff.
         */
        fetchIDPs(path)
            .then((idps) => {
            var _a;
            if (!idps.length || (idps.length === 1 && ((_a = idps[0]) === null || _a === void 0 ? void 0 : _a.id) === 'error')) {
                emitError();
                return;
            }
            this.emit('auth-fetch-idps-success', idps);
        })
            .catch(() => emitError());
    }
}
__decorate([
    AuthConfig
], DefaultAuthFormEntryStore.prototype, "config", void 0);
__decorate([
    CredentialManager
], DefaultAuthFormEntryStore.prototype, "credentialManager", void 0);
__decorate([
    FetchMachine
], DefaultAuthFormEntryStore.prototype, "fetchMachine", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "host", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "organisation", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "username", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "email", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "password", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "password2", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "showPasswords", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "hasPasswordExpired", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "passwordResetToken", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "message", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "hasErrors", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "mfaCode", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "idps", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "selectedIDP", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "ssoToggled", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "termsAgreed", void 0);
__decorate([
    observable
], DefaultAuthFormEntryStore.prototype, "autofilling", void 0);
__decorate([
    volatile
], DefaultAuthFormEntryStore.prototype, "orgUsername", null);
/**
 * DI key of the store.
 * @internal
 */
export const AuthFormEntryStore = registerStore(DefaultAuthFormEntryStore, 'AuthFormEntryStore');
